// src/index.ts
import { createGenerator } from "ts-json-schema-generator";
import { basename } from "path";

// src/consts.ts
var NAME = "vite-plugin-type-to-schema";
var ERROR_PREFIX = `[${NAME}]: `;

// src/dtsGenerator.ts
import { existsSync } from "fs";
import { readFile, writeFile } from "fs/promises";
function generateDtsCode(pluginConfig, files) {
  const body = Object.entries(files).map(([name, { exports }]) => {
    const content = exports.map((en) => `export const ${en}: JSONSchema;`).join("");
    return `declare module '*/${name}${pluginConfig.suffix}' { ${content} }
`;
  }).join("");
  return `// Generated by vite-plugin-type-to-schema plugin
// Don't modify this file

type JSONSchema = object;${""}

${body}
`;
}
async function generateDeclarationFile(pluginConfig, filepath, files) {
  const originalContent = existsSync(filepath) ? await readFile(filepath, "utf-8") : "";
  const code = generateDtsCode(pluginConfig, files);
  if (!code)
    return;
  if (code !== originalContent)
    await writeFile(filepath, code, "utf-8");
}

// src/resolveConfig.ts
import { resolve } from "path";
function resolveConfig(config = {}, root) {
  const dts = config.dts === false ? false : resolve(
    root,
    typeof config.dts === "string" ? config.dts : "schemas.d.ts"
  );
  return {
    suffix: config?.suffix || "?schema",
    options: config?.options || {},
    dts
  };
}

// src/index.ts
function handleFile(cfg) {
  let rootSchema;
  try {
    const generator = createGenerator(cfg);
    rootSchema = generator.createSchema(cfg.type);
  } catch (error) {
    throw new SyntaxError(`${ERROR_PREFIX}${error}`);
  }
  if (rootSchema == null) {
    throw new Error(`${ERROR_PREFIX}Incorrect TJS schema of "${cfg.path}"`);
  }
  if (!Object.keys(rootSchema.definitions).length) {
    throw new Error(`${ERROR_PREFIX}Schema "${cfg.path}" has no exported types`);
  }
  let code = "";
  const exportedNames = [];
  const resTable = {};
  for (const [schemaName, originalSchema] of Object.entries(rootSchema.definitions)) {
    const schemaVersion = rootSchema.$schema;
    const schema = JSON.parse(JSON.stringify(originalSchema));
    const defs = JSON.parse(JSON.stringify(rootSchema.definitions));
    delete defs[schemaName];
    Object.assign(schema, {
      $schema: schemaVersion,
      type: "object",
      definitions: defs
    });
    code += `export const ${schemaName} = ${JSON.stringify(schema)};`;
    exportedNames.push(schemaName);
    resTable[schemaName] = schema;
  }
  return { code, exportedNames };
}
function typeToSchema(config) {
  const root = process.cwd();
  const cfg = resolveConfig(config, root);
  const { suffix, options, dts } = cfg;
  const schemaFilesMeta = {};
  function onUpdate() {
    if (dts) {
      generateDeclarationFile(cfg, dts, schemaFilesMeta);
    }
    debugger;
  }
  return {
    name: NAME,
    load(id) {
      if (!id.endsWith(suffix))
        return;
      const purePath = id.replace(suffix, "");
      const fileName = basename(purePath, ".ts");
      debugger;
      const pathWithExt = purePath.endsWith(".ts") ? purePath : `${purePath}.ts`;
      const tjsgConfig = {
        ...options,
        path: pathWithExt,
        type: "*"
      };
      const result = handleFile(tjsgConfig);
      schemaFilesMeta[fileName] = { exports: result.exportedNames };
      debugger;
      return result.code;
    },
    buildEnd(err) {
      if (err)
        return;
      onUpdate();
    }
  };
}
export {
  typeToSchema as default
};
